#!/usr/bin/env perl

##
## USAGE: rnaclustGenReport.pl <dir>
##   where <dir> is the result directory of a mlocarna-p run
##        (we need the intermediateN.aln files, where N is a
##          running index on internal tree nodes in postorder)
##

use strict;

sub parse_clustal;
sub clean_clustal;

##------------------------------------------------------------
## options
use Getopt::Long;
use Pod::Usage;

# use MLocarna;

my $help;
my $man;

GetOptions(
	   "help"=> \$help,
	   "man" => \$man
	   ) || pod2usage(2);

pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;


## ----------------------------------------
# mfasta file

my $dir;
if ($#ARGV == 0) {
    $dir = $ARGV[0];
}else {
    pod2usage(1);
}

##------------------------------------------------------------


##
## originally mlocarna-p produces
## files intermediate<xyz>.aln
## we expect files $alnfile<xyz>.aln
my $intermediate_name_base="";

opendir(DIR,$dir);
my @files=readdir(DIR);
closedir DIR;
@files = grep /^$intermediate_name_base.*aln/, @files;

my $number_of_clusters=$#files+1;

#mkdir "Consensus";

## ---------------------------------------------------------------------------
# main program

my $header;

for (my $i=1; $i<=$number_of_clusters; $i++) {	
    #print STDERR "\n--------------------\n";
    #print STDERR "Cluster $i\n";
    
    my $alnfile = "$dir/$intermediate_name_base$i.aln";
    
    if ( ! -e $alnfile ) {   
	print STDERR "$alnfile does not exist\n";
	next;
    }

    my %aln = parse_clustal("$alnfile");
    my %clean_aln = clean_clustal(%aln);
    
    my @names = keys %clean_aln;
    my $clean_aln_length = length($clean_aln{$names[0]});
    
    if ($clean_aln_length == 0) {
    	print STDERR "  no local alignment\n";
    }
    else {
    	my $intermediate_name_clean=$alnfile;
    	$intermediate_name_clean =~ s/.aln$/.clean/;
    	$intermediate_name_clean .= ".aln";
    	
    	open(CLEAN,">$intermediate_name_clean");
    	#printf CLEAN "CLUSTAL -- cleaned $alnfile\n\n\n";
	printf CLEAN "$header\n\n";
    	for my $name (@names) {
    	    printf CLEAN "%-20s %s\n", $name,$clean_aln{$name};
    	}
	printf CLEAN "\n";
    	close CLEAN;
    #		
    #	my $size = $#names+1;
    #	
    #	my @rnazres=readpipe("RNAz $intermediate_name_clean");
    #	
    #	my $id;
    #	my $sci;
    #	
    #	for my $line (@rnazres) {
    #	    if ($line =~ /Mean pairwise identity:\s*(\S+)/) {
    #		$id = $1;
    #	    }
    #	    elsif ($line =~ /Structure conservation index:\s*(\S+)/) {
    #		$sci = $1;
    #	    }
    #	}
    #	
    #	my $score=`head -1 $alnfile | cut -d':' -f2`;
    #	
    #	$score =~ s/\s//g;
    #	chomp $score;
    #
    #	## compute consensus
    #	
    #	my @consensus = readpipe("RNAalifold -color $intermediate_name_clean 2>/dev/null");
    #	rename("alirna.ps","Consensus/$i.ps");
    #	open(CONS,">Consensus/$i.out") || die "Cannot write $!";
    #	for my $line (@consensus) {print CONS $line;}
    #	close CONS;
    #
    #	$consensus[1]=~/([.()]+)+\s+\(\s*([\d\.-]+)/ || die "$consensus[1]";
    #	
    #	my $cons_struct=$1;
    #	my $cons_mfe=$2;
    #	my @loops = ($cons_struct=~/(\(\.+\))/g);
    #	my $num_loops=$#loops+1;
    #	
    #	if (defined($sci) && defined($id)) {
    #	    # print STDERR "$i $size $score $id $sci $num_loops $cons_mfe $clean_aln_length\n";
    #	    print "$i $size $score $id $sci $num_loops $cons_mfe $clean_aln_length\n";
    #	}
    }
}


sub parse_clustal {
    my ($file)=@_;
    local *IN;

    my %res;
    #my $header;

    open(IN, $file) || die "Cannot read $file\n";
    
    my $line=<IN>;
    if ($line !~ /^CLUSTAL/) {
	print STDERR "$file is not a clustal file.\n";
	exit(-1);
    }else{
	$header=$line;

    }
    
    while ($line=<IN>) {
	#if($line =~ m/CLUSTAL/){
	 #   $header=$line;
	  #  print "HEAD: $header";
	#}else{
	    if ($line =~ /(\S+)\s+(\S+)/) {
		$res{$1}.=$2;
	    }
	#}
    }
    
    close (IN);

    return %res;
}


sub clean_clustal {
    my (%aln) = @_;

    my $threshold=0.4;

    my %clean;
    
    my @names=keys %aln;
    
    my $alisize = $#names+1;
    my $alilen = length($aln{$names[0]});
    
    my @count;
    for (my $i=0; $i<$alilen; $i++) {
	$count[$i]=0;
    }

    for my $name (@names) {
	my @seq=split //,$aln{$name};
	for (my $i=0; $i<$alilen; $i++) {
	    if ( $seq[$i] eq "~" ) {$count[$i]++;}
	}
    }
   
    for my $name (@names) {
	my @seq=split //,$aln{$name};
	my $newseq="";
	# printf "%-20s %s\n", "$name*",$aln{$name};
	#printf "%-20s ", $name;
	for (my $i=0; $i<$alilen; $i++) {
	    
	    if ( $count[$i] <= $threshold*$alisize ) {
		$newseq.=$seq[$i];
	    }
	    
	}
	$newseq =~ s/~/-/g;
	#print $newseq;
	#print "\n";
	$clean{$name} = $newseq;
    }


    return %clean;
}


__END__

=head1 NAME

rnaclustGenReport.pl

=head1 SYNOPSIS

rnaclustGenReport.pl <dir>

=head1 DESCRIPTION

generate a report from the multiple alignments cluster<n> in dir

=cut
